/*
 *
 *  * Copyright (C) 2024, KylinSoft Co., Ltd.
 *  *
 *  * This program is free software: you can redistribute it and/or modify
 *  * it under the terms of the GNU General Public License as published by
 *  * the Free Software Foundation, either version 3 of the License, or
 *  * (at your option) any later version.
 *  *
 *  * This program is distributed in the hope that it will be useful,
 *  * but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  * GNU General Public License for more details.
 *  *
 *  * You should have received a copy of the GNU General Public License
 *  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *  *
 *  * Authors: Nicole <buxiaoqing@kylinos.cn>
 *
 */
#ifndef CUSTOMTOOLTIP_H
#define CUSTOMTOOLTIP_H

#include <QDialog>
#include <QLabel>
#include <QBasicTimer>
#include <QGSettings>
#include <QDir>
#include <QSettings>
#include "common/common.h"
#include <mutex>

class CustomToolTip : public QDialog
{
    Q_OBJECT
public:
    CustomToolTip(const QString& text = QString(), QWidget *parent = nullptr);
    ~CustomToolTip();

    static CustomToolTip* self();

    enum PanelPosition {
        Bottom = 0, //!< The bottom side of the screen.
        Top,    //!< The top side of the screen.
        Left,   //!< The left side of the screen.
        Right   //!< The right side of the screen.
    };

    QString getText() const;
    void setText(const QString &text);

    Qt::Alignment getAlignment() const;
    void setAlignment(Qt::Alignment alignment);

    void anchorTarget(QWidget *target);
    const QLabel *label() const;
    void showTip(const QWidget *obj, int margin = 6);
    void showTip(const QPoint &rightBottom);
    void hideTip();
    void updateTipPostion(const QPoint &targetPoint);

    void opacityWatcher();

protected:
    void timerEvent(QTimerEvent *event) override;
    void paintEvent(QPaintEvent *event) override;

signals:
    void textChanged();
    void alignmentChanged();

private:
    QLabel *m_contentLabel = nullptr;
    QWidget *m_targetWidget = nullptr;
    QPoint m_targetPoint;
    QBasicTimer m_showTimer;
    QBasicTimer m_hideTimer;

    QGSettings *m_transparencyGsettings;
    double m_transparency;

    QGSettings *m_panelGsettings;
    int m_panelPosition = 0;
    int m_panelSize = 46;
};

#endif // CUSTOMTOOLTIP_H
