/*
 * Copyright (C) 2024 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU  Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */

#ifndef ABSTRACTCONFIGMONITOR_H
#define ABSTRACTCONFIGMONITOR_H

#include <QJsonObject>
#include <QMap>
#include <QObject>
#include <QString>

class AbstractConfigMonitor : public QObject
{
    Q_OBJECT

public:
    bool isKcmEnabled() const;

    bool isMainModuleContextMenuItemDisabledForcibly(const QString& menuItem) const;
    bool isMainModuleContextMenuItemCheckedForcibly(const QString& menuItem, bool defaultValue) const;
    bool isContextMenuItemDisabledForcibly(const QString& moduleName, const QString& menuItem) const;
    bool isContextMenuItemCheckedForcibly(const QString& moduleName, const QString& menuItem, bool defaultValue) const;

    const QString& getMainModuleName() const {
        return m_mainModuleName;
    }

    QStringList getSubmoduleNameList() const;

    QString getModuleContextMenuItemPolicyByName(const QString& moduleName, const QString& itemName) const;

    QString getMouseLeftButtonActionPolicy(const QString& moduleName) const;
    QString getMouseRightButtonActionPolicy(const QString& moduleName) const;

    QString getModuleControlPolicy(const QString& moduleName) const;
    QStringList getModuleControlList(const QString& moduleName, const QString& policyName) const;
    bool isSubmoduleWhitelistControlPolicyUsed(const QString& moduleName) const;
    bool isSubmoduleBlacklistControlPolicyUsed(const QString& moduleName) const;
    bool isSubmoduleInWhitelist(const QString& moduleName, const QString& submoduleName) const;
    bool isSubmoduleInBlacklist(const QString& moduleName, const QString& submoduleName) const;

protected:
    explicit AbstractConfigMonitor(const QString& configFileBaseName);

private:
    void loadAllModuleConfig();
    void connectUsdDBusSignal();

    QJsonObject getModuleConfigByName(const QString& moduleName) const;
    QJsonObject getMainModuleConfig() const;

    QString getModuleJsonFilePathByName(const QString& moduleName) const;
    QJsonObject createModuleConfigByName(const QString& moduleName) const;
    void loadModuleConfigByName(const QString &moduleName);
    void loadMainModuleConfig();
    void loadAllSubmoduleConfig();

    QString getMouseButtonActionPolicy(const QString& moduleName, const QString& btnName) const;

    QString getMainModuleControlPolicy() const;
    QString getSubmoduleControlPolicy(const QString& moduleName) const;

    QString getModuleNameFromFileName(const QString& fileName) const;
    bool shouldUpdateModuleConfig(const QString& fileName) const;

    bool isSubmoduleInPolicyList(const QString& moduleName, const QString& policyName,
        const QString& submoduleName) const;

private:
    const QString m_mainModuleName = "mainmodule";

    QMap<QString, QJsonObject> m_moduleConfigMap;

    const QString m_configFileBaseName;

    const QString m_whitelist = "white-list";
    const QString m_blacklist = "black-list";

private Q_SLOTS:
    void onConfigFileChanged(const QString& user, const QString& module, const QString& filePath);

Q_SIGNALS:
    void moduleConfigChanged(const QString& moduleName);
};

#endif
